from yankee.json import fields as f
from yankee.json import Schema


class DocumentStructureSchema(Schema):
    number_of_claims = f.Integer("numberOfClaims")
    number_of_drawing_sheets = f.Integer("numberOfDrawingSheets")
    number_of_figures = f.Integer("numberOfFigures")
    page_count = f.Integer("pageCount")
    front_page_end = f.Integer("frontPageEnd")
    front_page_start = f.Integer("frontPageStart")
    bib_start = f.Integer("bibStart")
    bib_end = f.Integer("bibEnd")
    abstract_start = f.Integer("abstractStart")
    abstract_end = f.Integer("abstractEnd")
    drawings_start = f.Integer("drawingsStart")
    drawings_end = f.Integer("drawingsEnd")
    description_start = f.Integer("descriptionStart")
    description_end = f.Integer("descriptionEnd")
    specification_start = f.Integer("specificationStart")
    specification_end = f.Integer("specificationEnd")
    claims_end = f.Integer("claimsEnd")
    claims_start = f.Integer("claimsStart")
    amend_start = f.Integer("amendStart")
    amend_end = f.Integer("amendEnd")
    cert_correction_end = f.Integer("certCorrectionEnd")
    cert_correction_start = f.Integer("certCorrectionStart")
    cert_reexamination_end = f.Integer("certReexaminationEnd")
    cert_reexamination_start = f.Integer("certReexaminationStart")
    ptab_start = f.Integer("ptabStart")
    ptab_end = f.Integer("ptabEnd")
    search_report_start = f.Integer("searchReportStart")
    search_report_end = f.Integer("searchReportEnd")
    supplemental_start = f.Integer("supplementalStart")
    supplemental_end = f.Integer("supplementalEnd")
